==> trircd-release-5.1.7-r.tar.gz: trircd-release-5.1.7-r/src/modules.c <==
/*
 *   Copyright (C) 2000-2002 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: modules.c,v 5.7 2002/08/11 11:17:00 tr-ircd Exp $
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <dirent.h>

#include <limits.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "confitem.h"
#include "h.h"
#include "hook.h"
#include "msg.h"
#include "numeric.h"
#include "modules.h"
#include "s_conf.h"

#ifndef STATIC_MODULES

#ifndef RTLD_NOW
#define RTLD_NOW RTLD_LAZY	/* openbsd deficiency */
#endif

#define MODS_INCREMENT 10

struct module **modlist = NULL;

static dlink_list mod_paths;

int num_mods = 0;
int max_mods = MODS_INCREMENT;

struct Message modload_t[] = {
    {MSG_MODLOAD, 0, 2, M_SLOW, 0L,
     m_unregistered, m_permission, m_modload, m_ignore, m_ignore}
};
struct Message modunload_t[] = {
    {MSG_MODUNLOAD, 0, 2, M_SLOW, 0L,
     m_unregistered, m_permission, m_modunload, m_ignore, m_ignore}
};
struct Message modreload_t[] = {
    {MSG_MODRELOAD, 0, 2, M_SLOW, 0L,
     m_unregistered, m_permission, m_modreload, m_ignore, m_ignore}
};
struct Message modres_t[] = {
    {MSG_MODRESTART, 0, 1, M_SLOW, 0L,
     m_unregistered, m_permission, m_modrestart, m_ignore, m_ignore}
};
struct Message modlist_t[] = {
    {MSG_MODLIST, 0, 1, M_SLOW, 0L,
     m_unregistered, m_permission, m_modlist, m_ignore, m_ignore}
};

struct Message modload_t_n[] = {
    {MSG_INSMOD, 0, 2, M_SLOW, 0L,
     m_unregistered, m_permission, m_modload, m_ignore, m_ignore}
};
struct Message modunload_t_n[] = {
    {MSG_RMMOD, 0, 2, M_SLOW, 0L,
     m_unregistered, m_permission, m_modunload, m_ignore, m_ignore}
};
struct Message modreload_t_n[] = {
    {MSG_DEPMOD, 0, 2, M_SLOW, 0L,
     m_unregistered, m_permission, m_modreload, m_ignore, m_ignore}
};

struct Message modlist_t_n[] = {
    {MSG_LSMOD, 0, 1, M_SLOW, 0L,
     m_unregistered, m_permission, m_modlist, m_ignore, m_ignore}
};

void init_modules(void)
{
    mod_add_cmd(modload_t);
    mod_add_cmd(modunload_t);
    mod_add_cmd(modreload_t);
    mod_add_cmd(modres_t);
    mod_add_cmd(modlist_t);
    mod_add_cmd(modload_t_n);
    mod_add_cmd(modunload_t_n);
    mod_add_cmd(modreload_t_n);
    mod_add_cmd(modlist_t_n);

    modlist = (struct module **) MyMalloc(sizeof(struct module) * (MODS_INCREMENT));

    max_mods = MODS_INCREMENT;
}

char *irc_basename(char *path)
{
    char *mod_basename;
    char *s;

    if (!path)
        return NULL;

    mod_basename = MyMalloc(strlen(path) + 1);

    if (!(s = strrchr(path, '/')))
	s = path;
    else
	s++;

    (void) strcpy(mod_basename, s);
    return mod_basename;
}

int findmodule_byname(char *name)
{
    int i;

    if (!name)
	return -1;

    for (i = 0; i < num_mods; i++) {
	if (!irc_strcmp(modlist[i]->name, name))
	    return i;
    }
    return -1;
}

/* load all modules from MODPATH & LANGPATH */
void load_all_modules(int which, int check)
{
    if (which & LM_MODULES)
    	load_modules_from(check, MODPATH);
    if (which & LM_LANG)
        load_modules_from(check, LANGPATH);
    if (which & LM_CHANMODES)
    	load_modules_from(check, CHANMODEPATH);
}

void load_modules_from(int check, char *frompath)
{
    DIR *system_module_dir = NULL;
    struct dirent *ldirent = NULL;
    char module_fq_name[PATH_MAX + 1];

    mod_add_path(frompath);

    system_module_dir = opendir(frompath);

    if (system_module_dir == NULL) {
	printf("Could not load modules from %s: %s", frompath, strerror(errno));
	return;
    }
    while ((ldirent = readdir(system_module_dir)) != NULL) {
	if (ldirent->d_name[strlen(ldirent->d_name) - 3] == '.' &&
	    ldirent->d_name[strlen(ldirent->d_name) - 2] == 's' &&
	    ((ldirent->d_name[strlen(ldirent->d_name) - 1] == 'o') || 
	     (ldirent->d_name[strlen(ldirent->d_name) - 1] == 'l'))) {
	    (void) sprintf(module_fq_name, "%s/%s", frompath, ldirent->d_name);
	    (void) load_a_module(module_fq_name, check);
	}
    }
    (void) closedir(system_module_dir);

}

void load_dependencies() {

    int i;

    for (i = 0; i < num_mods; i++) {
        if (modlist[i]->dependent) 
	    deinit_reinit_module(modlist[i]->name);
    }
}

int load_one_module(char *path)
{
    char modpath[MAXPATHLEN];
    dlink_node *pathst;
    struct module_path *mpath;
    struct stat statbuf;

    if (strchr(path, '/'))	/* absolute path, try it */
	return load_a_module(path, 1);

    for (pathst = mod_paths.head; pathst; pathst = pathst->next) {
	mpath = (struct module_path *) pathst->data;

	sprintf(modpath, "%s/%s", mpath->path, path);
	if (stat(modpath, &statbuf) == 0)
	    return load_a_module(modpath, 1);
    }

    sendto_ops("Cannot locate module %s", path);
    return -1;
}

static struct module_path *mod_find_path(char *path)
{
    dlink_node *pathst = mod_paths.head;
    struct module_path *mpath;

    if (!pathst)
	return NULL;

    for (; pathst; pathst = pathst->next) {
	mpath = (struct module_path *) pathst->data;

	if (!strcmp(path, mpath->path))
	    return mpath;
    }

    return NULL;
}

void mod_add_path(char *path)
{
    struct module_path *pathst;
    dlink_node *node;

    if (mod_find_path(path))
	return;

    pathst = (struct module_path *) MyMalloc(sizeof(struct module_path));
    node = make_dlink_node();

    strcpy(pathst->path, path);
    dlinkAdd(pathst, node, &mod_paths);
}

void mod_clear_paths(void)
{
    struct module_path *pathst;
    dlink_node *node, *next;

    for (node = mod_paths.head; node; node = next) {
	next = node->next;
	pathst = (struct module_path *) node->data;
	dlinkDelete(node, &mod_paths);
	free_dlink_node(node);
	MyFree(pathst);
    }
}

int m_modload(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    char *m_bn;
    if (!IsAdmin(sptr))
	return m_permission(cptr, sptr, parc, parv);
    m_bn = irc_basename(parv[1]);
    if (findmodule_byname(m_bn) != -1) {
	send_me_notice(sptr, ":Module %s is already loaded", m_bn);
	return 0;
    }

    (void) load_one_module(parv[1]);
    MyFree(m_bn);
    return 0;
}

int m_modunload(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    char *m_bn;
    if (!IsAdmin(sptr))
	return m_permission(cptr, sptr, parc, parv);
    m_bn = irc_basename(parv[1]);
    if (findmodule_byname(m_bn) == -1) {
	send_me_notice(sptr, ":Module %s is not loaded", m_bn ? m_bn : "");
	MyFree(m_bn);
	return 0;
    }
    if (strstr(m_bn, "cm_")) {
	send_me_notice(sptr, ":Channelmode modules cannot be unloaded");
	MyFree(m_bn);
	return 0;
    }

    if (unload_one_module(m_bn, 1) == -1)
	send_me_notice(sptr, ":Module %s is not loaded", m_bn);
    MyFree(m_bn);
    return 0;
}

int m_modreload(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    char *m_bn;
    if (!IsAdmin(sptr))
	return m_permission(cptr, sptr, parc, parv);
    m_bn = irc_basename(parv[1]);
    if (findmodule_byname(m_bn) == -1) {
	send_me_notice(sptr, ":Module %s is not loaded", m_bn ? m_bn : "");
	MyFree(m_bn);
	return 0;
    }

    if (unload_one_module(m_bn, 1) == -1) {
	send_me_notice(sptr, ":Module %s is not loaded", m_bn);
	MyFree(m_bn);
	return 0;
    }

    (void) load_one_module(parv[1]);
    MyFree(m_bn);
    return 0;
}

int m_modlist(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    int i;
    if (!IsAdmin(sptr))
	return m_permission(cptr, sptr, parc, parv);
    if (parc > 1) {
	send_me_notice(sptr, ":Listing modules matching string '%s'...", parv[1]);
    } else {
	send_me_notice(sptr, ":Listing all modules...");
    }

    for (i = 0; i < num_mods; i++) {
	if (parc > 1) {
	    if (strstr(modlist[i]->name, parv[1])) {
		send_me_numeric(sptr, RPL_MODLIST,
				modlist[i]->name, modlist[i]->address, modlist[i]->version);
	    }
	} else {
	    send_me_numeric(sptr, RPL_MODLIST,
			    modlist[i]->name, modlist[i]->address, modlist[i]->version);
	}
    }
    send_me_notice(sptr, ":Done.");
    return 0;
}

int m_modrestart(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    int i, modnum;
    if (!IsAdmin(sptr))
	return m_permission(cptr, sptr, parc, parv);
    send_me_notice(sptr, ":Reloading all modules");
    modnum = num_mods;

    for (i = num_mods; i; i--) {
	if (modlist[i] && (modlist[i]->persistent == 0))
	    unload_one_module(modlist[0]->name, 0);
    }

    load_all_modules(LM_MODULES|LM_LANG, 0);
    rehash(&me, &me, 1);
    sendto_ops("Module Restart: %d modules unloaded, %d modules loaded", modnum, num_mods);
    return 0;
}

#else

void init_modules(void) 
{
    return;
}

void load_dependencies() {

    return;
}

void load_all_modules(int which, int check)
{
    if (which & LM_MODULES)
    	static_modules_init();
    if (which & LM_LANG)
    	static_languages_init();
    if (which & LM_CHANMODES)
    	static_chanmode_modules_init();
    if (which & LM_CONTRIB)
    	static_contrib_init();
}

#endif /* STATIC_MODULES */

==> trircd-release-5.7.3-r.tar.gz: trircd-release-5.7.3-r/src/modules.c <==
/************************************************************************
 *   IRC - Internet Relay Chat, src/modules.c
 *
 *   Copyright (C) 2000-2003 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: modules.c,v 1.3 2003/06/14 13:55:52 tr-ircd Exp $
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "confitem.h"
#include "h.h"
#include "hook.h"
#include "msg.h"
#include "numeric.h"
#include "modules.h"
#include "s_conf.h"

#ifndef STATIC_MODULES

struct module *findmodule_byname(char *name)
{
    dlink_node *ptr;
    struct module *mod;

    if (!name)
	return NULL;

    for (ptr = loaded_module_list.head; ptr; ptr = ptr->next) {
	mod = ptr->data;
	if (!mod)
	    continue;
	if (!irc_strcmp(mod->name, name))
	    return mod;
    }
    return NULL;
}

/* load all modules from MODPATH & LANGPATH */
void load_all_modules(int which, int check)
{
    if (which & LM_MODULES)
    	load_modules_from(check, MODPATH);
    if (which & LM_LANG)
        load_modules_from(check, LANGPATH);
    if (which & LM_CHANMODES)
    	load_modules_from(check, CHANMODEPATH);
    mod_add_path(TOOLMODPATH);
}

void load_modules_from(int check, char *frompath)
{
    DIR *system_module_dir = NULL;
    struct dirent *ldirent = NULL;
    char module_fq_name[PATH_MAX + 1];

    mod_add_path(frompath);

    system_module_dir = opendir(frompath);

    if (system_module_dir == NULL) {
	printf("Could not load modules from %s: %s", frompath, strerror(errno));
	return;
    }
    while ((ldirent = readdir(system_module_dir)) != NULL) {
	if (ldirent->d_name[strlen(ldirent->d_name) - 3] == '.' &&
	    ldirent->d_name[strlen(ldirent->d_name) - 2] == 's' &&
	    ((ldirent->d_name[strlen(ldirent->d_name) - 1] == 'o') || 
	     (ldirent->d_name[strlen(ldirent->d_name) - 1] == 'l'))) {
	    sprintf(module_fq_name, "%s/%s", frompath, ldirent->d_name);
	    enqueue_one_module_to_load(module_fq_name, MODULE_AUTOLOAD);
	}
    }
    closedir(system_module_dir);

}

#else

static int did_once = 0;

void load_all_modules(int which, int check)
{
    if (did_once == 0) {
	did_once = 1;
    	static_modules_init();
    }
}

#endif /* STATIC_MODULES */
